/************************************************************************/
/*									*/
/*	vartest.c							*/
/*									*/
/*	Copyright (c) 1992, Research Systems Inc.  All rights reserved.	*/
/*									*/
/*	Test all variable types can be sent and received properly.	*/
/*	Check for memory leaks/temporary variable release as well.	*/
/*									*/
/************************************************************************/


/*	All IDL server clients need to include 'idlrpc.h'		*/

#include        <stdio.h>
#include        <ctype.h>
#include        <rpc/rpc.h>
#include	"export.h"
#include	"rpc_idl.h"
#include	"string.h"

CLIENT		* client;	/*	opaque connection mechanism	*/

main(argc, argv)
     int	argc;
     char	*argv[];
{
    char	*hostname	= NULL;
    
    /*	Set up connection with IDL server. If there is a command	*/
    /*	line argument, assume that it is the hostname where the IDL	*/
    /*	server is running.						*/

    if(argc == 2)
	hostname	= argv[1];
    
    client	= register_idl_client( 0, hostname, NULL );
    
    /*	if client is NULL, then there is no IDL server running		*/

    if( client == NULL ) {
	printf("Can't find server.\n");
	exit(1);
    }

    /*	Perform testing with scalar variables.				*/

    send_scalars();
    get_scalars();
    loop_with_scalars();

    /*	Perform testing with array variables				*/

    send_arrays();
    get_arrays();
    loop_with_arrays();

    /*	Disconnect from server						*/

    unregister_idl_client( client );
}


static	void set_var(var)
    varinfo_t	* var;
{
    if( set_idl_variable( client, var ) != 1 ) {
	printf("set_idl_variable for %s failed\n", var->Name );
	exit(1);
    }
}

static	void get_var(name, var)
    char	* name;
    varinfo_t	* var;
{
    if( get_idl_variable( client, name, var, 0 ) != 1 ) {
	printf("get_idl_variable for %s failed\n", name );
	exit(1);
    }
}

static	void	send_cmd(cmd)
    char	*cmd;
{
    int		ret_code;
    if( (ret_code = send_idl_command( client, cmd )) != 1 ) {
	printf("Error %d encountered processing command \"%s\"\n",
	       ret_code, cmd );
    }
}

/************************************************************************/
/*									*/
/*	send_scalars							*/
/*									*/
/*	Create variables on client side and send them to IDL.		*/
/*	Create one variable (A) which changes its type. Create		*/
/*	new variable of each type as well. Types created include	*/
/*	byte,short(int),long,float,double,complex,string, and undefined	*/
/*									*/
/************************************************************************/

send_scalars()
{
    varinfo_t	varA, varN;
    

    /*	Make note of what we are doing					*/
    
    printf("-----------Sending Variables--------------\n\n");

    /*	Clear varinfo_t structures before using them			*/

    bzero( &varA, sizeof(varA));
    if( !v_ensure_vptr( &varA ) )
	exit(1);
    bzero( &varN, sizeof(varN));
    if( !v_ensure_vptr( &varN ) )
	exit(1);

    /*	Create a variable, 'A' of void type and value			*/

    strcpy(varA.Name, "A" );
    varA.Variable->type	= TYP_UNDEF;
    set_var( &varA );
    send_cmd("help,a");

    /*	A = 65b, B = 65b	-- create Byte variables		*/

    if( v_make_byte( &varA, "a", 65 ) == 0 ) exit(1);
    if( v_make_byte( &varN, "b", 65 ) == 0 ) exit(1);

    /*	Send variables							*/

    set_var( &varA );
    set_var( &varN );

    /*	See if they arrived and are what we expect them to be.		*/
    
    send_cmd("help,a");
    send_cmd("help,b");


    /*	create (IDL) integer variables.					*/
    
    if( v_make_int( &varA, "a", 12345 ) == 0) exit(1);
    if( v_make_int( &varN, "c", 12345 ) == 0) exit(1);
    set_var( &varA );
    set_var( &varN );
    send_cmd("help,a");
    send_cmd("help,c");


    /*	Create long integer variables					*/
    
    if( v_make_long( &varA, "a", 1234567890L ) == 0) exit(1);
    if( v_make_long( &varN, "D", 5L ) == 0) exit(1);
    set_var( &varA );
    set_var( &varN );
    send_cmd("help,a");
    send_cmd("help,d");


    /*	Create (4 byte) floating point values				*/

    if( v_make_float( &varA, "a", 1.0 ) == 0) exit(1);
    if( v_make_float( &varN, "VERYVERYLONGVARIABLENAME", 123.456 ) == 0)
	exit(1);
    set_var( &varA );
    set_var( &varN );
    send_cmd("help,a");
    send_cmd("help,veryverylongvariablename");

    /*	Create (8 byte) double precision floating point values		*/
    
    if( v_make_double( &varA, "a", 1e6 ) == 0) exit(1);
    if( v_make_double( &varN, "e", 1e6 ) == 0) exit(1);
    set_var( &varA );
    set_var( &varN );
    send_cmd("help,a");
    send_cmd("help,e");

    /*	Create string variables						*/
    
    if( v_make_string( &varA, "a", "Hello World" ) == 0) exit(1);
    if(v_make_string(&varN, "f", "Now is the time for all good men...") == 0)
	exit(1);
    set_var( &varA );
    set_var( &varN );
    send_cmd("help,a");
    send_cmd("help,f");

    /*	Create complex (real component,imaginary component) values	*/
    
    if( v_make_complex( &varA, "a", 1.234, 5.678 ) == 0) exit(1);
    if( v_make_complex( &varN, "g", 3.0, 4.0 ) == 0) exit(1);
    set_var( &varA );
    set_var( &varN );
    send_cmd("help,a");
    send_cmd("help,g");

    /*	show current value of variables A through G			*/

    send_cmd("print,a,b,c,d,e,f,g" );
}


/************************************************************************/
/*									*/
/*	print_var							*/
/*									*/
/*	print contents of a variable.  Print only the first 10 elements	*/
/*	of array variables. This function only handles LONG,FLOAT,	*/
/*	DOUBLE and STRING arrays. It could easily be extented.		*/
/*									*/
/************************************************************************/

print_var( v )
varinfo_t	*v;
{
    VARIABLE	*var;
    double	*d;
    float	*f;
    LONG	*l;
    STRING	*s;
    int		i;

    static	char *types[] = { "Undef", "Byte", "Int", "Long",
				      "Float", "Double", "Complex", "String",
				      "Struct???" };


    /*	Print the name and type of the variable				*/
    
    printf("Name: %s\n", v->Name );
    var	= v->Variable;
    printf("Type = %s\n", types[var->type ] );

    printf("Value = ");

    /*	If its an array, print the first 10 elements			*/
    /*	Note: This will do bad things if the array doesn't have at	*/
    /*	least 10 elements.						*/
    /*	This only prints a subset of the available types		*/
    
    if( var->flags & V_ARR ) {

	switch( var->type ) {
	  case TYP_LONG:
	    for( l = (LONG*)var->value.arr->data, i = 0; i < 10; ++i )
		printf(" %ld", l[i] );
	    break;
	  case TYP_FLOAT:
	    for( f = (float*)var->value.arr->data, i = 0; i < 10; ++i )
		printf(" %f", f[i] );
	    break;
	  case TYP_DOUBLE:
	    for( d = (double*)var->value.arr->data, i = 0; i < 10; ++i )
		printf(" %lf", d[i] );
	    break;
	  case TYP_STRING:
	    for( s = (STRING*)var->value.arr->data, i = 0; i < 10; ++i ) {
		printf("(%x)", s[i].s );
		printf(" %s", STRING_STR( &s[i] ) );
	    }
	    break;
	  default:
	    printf("print_var: can't print arrays of type %d",var->type);
	}
	printf("...\n" );
    }

    /*	Otherwise, print the value. UNDEF's have no value		*/
    else {
	switch( var->type ) {

	  case TYP_UNDEF:	printf("<Nil>\n");		break;
	  case TYP_BYTE:	printf("%d\n", var->value.c );	break;
	  case TYP_INT:		printf("%d\n", var->value.i );	break;
	  case TYP_LONG:	printf("%ld\n", var->value.l );	break;
	  case TYP_FLOAT:	printf("%f\n", var->value.f );	break;
	  case TYP_DOUBLE:	printf("%lf\n", var->value.d );	break;
	  case TYP_COMPLEX:	printf("(%f %f)\n",
				        var->value.cmp.r, var->value.cmp.i );
	    break;
	  case TYP_STRING:	printf("%s\n", STRING_STR(&var->value.str) );
	    break;
	  default:
	    printf("<unprintable>\n");	/* Struct???			*/
	    break;
	}
    }
}


/************************************************************************/
/*									*/
/*	get_scalars							*/
/*									*/
/*	Attempt to read in the variables we created in send_scalars	*/
/*	Attempt to read a non existent variable as well. Print		*/
/*	the value and type of what we got to verify correctness.	*/
/*									*/
/************************************************************************/

get_scalars()
{
    varinfo_t	var;

    bzero( &var, sizeof(varinfo_t));

    printf("-----------Getting Variables--------------\n\n");

    if( get_idl_variable( client, "a", &var, 0 ) != 1) exit(2);
    print_var( &var );

    if( get_idl_variable( client, "b", &var, 0 ) != 1) exit(2);
    print_var( &var );

    if( get_idl_variable( client, "c", &var, 0 ) != 1) exit(2);
    print_var( &var );

    if( get_idl_variable( client, "d", &var, 0 ) != 1) exit(2);
    print_var( &var );

    if( get_idl_variable( client, "e", &var, 0 ) != 1) exit(2);
    print_var( &var );

    if( get_idl_variable( client, "f", &var, 0 ) != 1) exit(2);
    print_var( &var );

    if( get_idl_variable( client, "g", &var, 0 ) != 1) exit(2);
    print_var( &var );

    if( get_idl_variable( client, "foobie", &var, 0 ) != 1) exit(2);
    print_var( &var );
    free_idl_var( &var );
}


/************************************************************************/
/*									*/
/*	loop_with_scalars						*/
/*									*/
/*	Create a variable and set it in the server 1000 times.		*/
/*	Verify that no memory leaks occur.				*/
/*									*/
/************************************************************************/

loop_with_scalars()
{
    int		i;
    varinfo_t	var;

    printf("-----------Loop with Scalars--------------\n\n");
    bzero( &var, sizeof(var));

    v_make_float( &var, "_Fubar", 9.5 );
    set_var( &var );

    send_cmd( "help,/mem");
    for(i = 0; i < 1000; ++i )
	set_var( &var );
    send_cmd( "help,/mem");
}


/************************************************************************/
/*	Array variable declarations					*/
/************************************************************************/

varinfo_t	arr1;		/* arr1 = float(dim1)			*/
varinfo_t	arr2;		/* arr2	= string(dim2)			*/
varinfo_t	arr3;		/* arr3	= double(dim3)			*/
varinfo_t	arr4;		/* arr4	= long(dim4)			*/

/*	Array dimension declarations					*/
    
LONG		dim1[1] = { 10000 };
LONG		dim2[1] = { 10 };
LONG		dim3[3] = { 4, 5, 5 };
LONG		dim4[6] = { 2, 2, 2, 2, 2, 2 };

/*	Array contents declarations					*/

float		buf1[10000];
STRING		buf2[10];
double		buf3[100];
LONG		buf4[64];


#ifdef ultrix
/* Some systems lack the standard strdup(3) function. */
char *strdup(s)
     char *s;
{
  extern char *malloc();
  char *ret;
  unsigned int len;

  len = strlen(s) + 1;
  if (ret = malloc(len)) bcopy(s, ret, len);
  return ret;
}
#endif



/************************************************************************/
/*									*/
/*	send_arrays							*/
/*									*/
/*	Create 4 array variables of different sorts and send them	*/
/*	to the server.							*/
/*									*/
/************************************************************************/

send_arrays()
{
    char	*p, *q;
    int		i;

    printf("-----------Sending Array Variables--------------\n\n");

    /*	Clear the varinfo_t structures.					*/
    
    bzero( &arr1, sizeof(arr1));
    bzero( &arr2, sizeof(arr2));
    bzero( &arr3, sizeof(arr3));
    bzero( &arr4, sizeof(arr4));


    /********************************************************************/
    /*	ARR1	= FINDGEN(10000)					*/
    /********************************************************************/

    bzero(&arr1, sizeof(arr1));

    /*	Generate array data contents					*/

    for( i = 0; i < 10000; ++i )
	buf1[i]	= (float)i;

    /*	Fill varinfo_t structure					*/
    
    if( !v_fill_array( &arr1, "Arr1", TYP_FLOAT, 1, dim1, (UCHAR*)buf1, 0) )
	printf("v_fill_array arr1 failed\n"), exit(1);

    /*	Send to server							*/
    
    set_var( &arr1);


    /********************************************************************/
    /*	arr2 = ( "aaaaaaaa", "bbbbbbbb", ... "jjjjjjjj" )		*/
    /********************************************************************/

    bzero(&arr2, sizeof(arr2));

    for( i =0; i < 10; ++i ) {
	bzero(&buf2[i], sizeof(STRING));
	q	= p	= strdup("aaaaaaaa");
	if( q == NULL ) {
	    printf("Ran out of memory in send_array");
	    exit(1);
	}

	/*	Change all the letters from 'a' to the correct		*/
	/*	value for the given element.				*/
	
	while(*q) *q++ += i;
	v_fill_string( &buf2[i], p );
    }
    if( !v_fill_array( &arr2, "Arr2", TYP_STRING, 1, dim2, (UCHAR*)buf2, 0) )
	printf("v_fill_array arr2 failed\n"), exit(1);
    set_var( &arr2);

    /********************************************************************/
    /*	arr3 = 1.0/(dindgen(4,5,5)+1.0)					*/
    /********************************************************************/

    bzero(&arr3, sizeof(arr3));
    for( i = 1; i < 101; ++i ) {
	buf3[i-1]	= 1.0 / (double)i;
    }

    if( !v_fill_array( &arr3, "Arr3", TYP_DOUBLE, 3, dim3, (UCHAR*)buf3, 0) )
	printf("v_fill_array arr3 failed\n"), exit(1);
    set_var( &arr3);

    /********************************************************************/
    /*	arr4 = 64-lindgen(2,2,2,2,2,2)					*/
    /********************************************************************/

    bzero(&arr4, sizeof(arr4));
    for( i = 0; i < 64; ++i ) {
	buf4[i]	= 64 - i;
    }
    if( !v_fill_array( &arr4, "Arr4", TYP_LONG, 6, dim4, (UCHAR*)buf4, 0) )
	printf("v_fill_array arr4 failed\n"), exit(1);
    set_var( &arr4);


    /*	Examine the variables that we created in the server		*/
    
    send_cmd( "help,arr1");
    send_cmd( "help,arr2");
    send_cmd( "help,arr3");
    send_cmd( "help,arr4");
}


/************************************************************************/
/*									*/
/*	get_arrays							*/
/*									*/
/*	Attempt to read in the variables we created in send_arrays	*/
/*	Print the value and type of what we got to verify correctness.	*/
/*									*/
/************************************************************************/

get_arrays()
{
    varinfo_t	var;

    printf("-----------Getting Array Variables--------------\n\n");


    /*	Read in the variable. get_idl_variable will return 1 on		*/
    /*	success, 0 for communication failure, and -1 if our		*/
    /*	request was malformed						*/

    bzero(&var, sizeof(varinfo_t));
    if( get_idl_variable( client, "ARR1", &var, 0 ) != 1 ) {
	printf("get_arrays: fetch of ARR1 failed");
	exit(1);
    }

    /*	Display the value gotten					*/
    print_var( &var );

    /*	Read in ARR2							*/
    
    if( get_idl_variable( client, "ARR2", &var, 0 ) != 1 ) {
	printf("get_arrays: fetch of ARR2 failed");
	exit(1);
    }
    print_var( &var );

    /*	Read in ARR3							*/
    
    if( get_idl_variable( client, "ARR3", &var, 0 ) != 1 ) {
	printf("get_arrays: fetch of ARR3 failed");
	exit(1);
    }
    print_var( &var );

    /*	Read in ARR4							*/
    
    if( get_idl_variable( client, "ARR4", &var, 0 ) != 1 ) {
	printf("get_arrays: fetch of ARR4 failed");
	exit(1);
    }
    print_var( &var );
    free_idl_var( &var );
}


/************************************************************************/
/*									*/
/*	loop_with_arrays						*/
/*									*/
/*	send at random ARR1,ARR2,ARR3 or ARR4. Send 1000 times and	*/
/*	check for memory leaks (in server)				*/
/*									*/
/************************************************************************/

loop_with_arrays()
{
    int		i;
    int		varno;

    printf("-----------Loop with Arrays--------------\n\n");

    send_cmd("help,/mem");

    /*	Initialize random number generator. Note: rand() is a vary	*/
    /*	poor random number generator. Would use random(3) but it gets	*/
    /*	called different things on different machines (E.g. random48)	*/
    /*	and this would reduce the portability of this demo		*/
    
    srand(getpid());

    /*	Loop 1000 times send variables at random			*/
    
    for( i = 0; i < 1000; ++i ) {
	/*	choose a quasi-random number from 0-3	*/
	switch( varno = (rand() >> 9) & 3 ) {
	  case 0:	set_var( &arr1 );	break;
	  case 1:	set_var( &arr2 );	break;
	  case 2:	set_var( &arr3 );	break;
	  case 3:	set_var( &arr4 );	break;
	}
	/*	Tell user which variable we sent			*/
	printf("%d ", varno+1 );
	fflush(stdout);
    }
    putchar('\n');

    /*	Check for memory loss						*/
    
    send_cmd("help,/mem");
}

